@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2018 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2019 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
@@ -77,8 +77,10 @@ def error(message, *args):
 def need_conversion(dst_filename, data_dict, source_filename):
     with open(source_filename, 'rb') as f:
         blob = f.read(4096)
-        version = int(re.search(b'version number="\\$Revision: (\\d+)',
-                                blob).group(1))
+        version_match = re.search(b'version number="\\$Revision: (\\d+)', blob)
+        if not version_match:  # CLDR 36.0 was shipped without proper revision numbers
+            return True
+        version = int(version_match.group(1))
 
     data_dict['_version'] = version
     if not os.path.isfile(dst_filename):
@@ -423,6 +425,7 @@ def _process_local_datas(sup, srcdir, destdir, force=False, dump_json=False):
         parse_percent_formats(data, tree)
 
         parse_currency_formats(data, tree)
+        parse_currency_unit_patterns(data, tree)
         parse_currency_names(data, tree)
         parse_unit_patterns(data, tree)
         parse_date_fields(data, tree)
@@ -519,18 +522,26 @@ def parse_dates(data, tree, sup, regions, territory):
     week_data = data.setdefault('week_data', {})
     supelem = sup.find('.//weekData')
     for elem in supelem.findall('minDays'):
+        if _should_skip_elem(elem):
+            continue
         territories = elem.attrib['territories'].split()
         if territory in territories or any([r in territories for r in regions]):
             week_data['min_days'] = int(elem.attrib['count'])
     for elem in supelem.findall('firstDay'):
+        if _should_skip_elem(elem):
+            continue
         territories = elem.attrib['territories'].split()
         if territory in territories or any([r in territories for r in regions]):
             week_data['first_day'] = weekdays[elem.attrib['day']]
     for elem in supelem.findall('weekendStart'):
+        if _should_skip_elem(elem):
+            continue
         territories = elem.attrib['territories'].split()
         if territory in territories or any([r in territories for r in regions]):
             week_data['weekend_start'] = weekdays[elem.attrib['day']]
     for elem in supelem.findall('weekendEnd'):
+        if _should_skip_elem(elem):
+            continue
         territories = elem.attrib['territories'].split()
         if territory in territories or any([r in territories for r in regions]):
             week_data['weekend_end'] = weekdays[elem.attrib['day']]
@@ -755,7 +766,7 @@ def parse_decimal_formats(data, tree):
                 pattern = numbers.parse_pattern(text_type(pattern_el.text))
                 if pattern_type:
                     # This is a compact decimal format, see:
-                    # http://www.unicode.org/reports/tr35/tr35-45/tr35-numbers.html#Compact_Number_Formats
+                    # https://www.unicode.org/reports/tr35/tr35-45/tr35-numbers.html#Compact_Number_Formats
 
                     # These are mapped into a `compact_decimal_formats` dictionary
                     # with the format {length: {count: {multiplier: pattern}}}.
@@ -860,7 +871,7 @@ def parse_date_fields(data, tree):
 
 
 def parse_interval_formats(data, tree):
-    # http://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
+    # https://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
     interval_formats = data.setdefault("interval_formats", {})
     for elem in tree.findall("dateTimeFormats/intervalFormats/*"):
         if 'draft' in elem.attrib:
@@ -903,6 +914,17 @@ def parse_currency_formats(data, tree):
                         currency_formats[type] = numbers.parse_pattern(pattern)
 
 
+def parse_currency_unit_patterns(data, tree):
+    currency_unit_patterns = data.setdefault('currency_unit_patterns', {})
+    for currency_formats_elem in tree.findall('.//currencyFormats'):
+        if _should_skip_number_elem(data, currency_formats_elem):  # TODO: Support other number systems
+            continue
+        for unit_pattern_elem in currency_formats_elem.findall('./unitPattern'):
+            count = unit_pattern_elem.attrib['count']
+            pattern = text_type(unit_pattern_elem.text)
+            currency_unit_patterns[count] = pattern
+
+
 def parse_day_period_rules(tree):
     """
     Parse dayPeriodRule data into a dict.
