@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2018 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2019 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
@@ -13,7 +13,6 @@
 
 from datetime import datetime
 import unittest
-import sys
 
 from babel.core import Locale
 from babel.messages.catalog import Catalog, Message
@@ -456,16 +455,8 @@ msgstr[2] "Vohs [text]"
             '''
         # Catalog not created, throws Unicode Error
         buf = StringIO(invalid_po)
-        output = None
-
-        # This should only be thrown under py27
-        if sys.version_info.major == 2:
-            with self.assertRaises(UnicodeEncodeError):
-                output = pofile.read_po(buf, locale='fr', abort_invalid=False)
-            assert not output
-        else:
-            output = pofile.read_po(buf, locale='fr', abort_invalid=False)
-            assert isinstance(output, Catalog)
+        output = pofile.read_po(buf, locale='fr', abort_invalid=False)
+        assert isinstance(output, Catalog)
 
         # Catalog not created, throws PoFileError
         buf = StringIO(invalid_po_2)
